#include <fstream>
#include <iostream>
//#include <sys/sysinfo.h>

#include "raw-read.h"
#include "read-processor.h"
#include "utils-file.h"
#include "utils-string.h"

using namespace std;

int ReadProcessor::frecs_[fc_];
int ReadProcessor::fmer_[fc_];
int ReadProcessor::total_;
int ReadProcessor::from_val_;
int ReadProcessor::to_val_;
int ReadProcessor::cov_;
string ReadProcessor::fname_[fc_];
fstream* ReadProcessor::fp_[fc_];
int ReadProcessor::used_[NUM_PAIR];
int ReadProcessor::num_used_[NUM_PAIR];
int ReadProcessor::prev_used_[NUM_PAIR];
IntIntMtTripleVec ReadProcessor::inmem_files_[fc_];
vector<RawRead> ReadProcessor::raw_read_;

ReadProcessor::ReadProcessor(const string& path) {
  fill(used_, used_ + NUM_PAIR, -1);
  fill(num_used_, num_used_ + NUM_PAIR, 0);
  fill(prev_used_, prev_used_ + NUM_PAIR, -1);

  // This file contains all the raw-paired reads.
  fstream* pairs = UtilsFile::OpenFile(path, "r");
  
  // We are assumming all reads are preceeded by an id or some other info.
  total_ = 0;
  cov_ = 0;
  string ida, idb;
  while ((*pairs) >> ida) {
    string lmar, rmar;
    //    cout << id << ":" << total_ << endl;
    (*pairs) >> lmar >> idb >> rmar;
    cov_ += lmar.size();
    cov_ += rmar.size();
    // cout << UtilsString::Fasta2Solid(lmar) << endl;
    //cout << id << ":" << total_ << endl;
    //cout << UtilsString::Fasta2Solid(rmar) << endl;


    // At present, doing nothing with the id, qual score.
    vector<int> lqual, rqual;
#ifdef COMPRESS
    lmar = UtilsString::CompressStr(lmar);
    rmar = UtilsString::CompressStr(rmar);
#endif  // COMPRESS
    
    // Ignore the not-useful pairs.
    if (lmar.size() < MIN_READ_LEN || rmar.size() < MIN_READ_LEN) continue;

    // Sometimes we use a portion of the reads.
    //if (rand() % 100 < USE_READ_PORTION) continue;
    //    if (rand() % USE_READ_PORTION) continue;
    raw_read_.push_back(RawRead(lmar, rmar, ida, idb, lqual, rqual, total_++));
    for (int i = 0; i < MAR_TYPES; ++i) {
      FreqCount(raw_read_[raw_read_.size() - 1].Mar((mar_types)i)); 
    }
  }
#ifdef DEBUG
  cout << "total # of pairs: " << total_ << endl;
#endif  // DEBUG
  UtilsFile::CloseFile(pairs);

  for (int i = 0; i < fc_; ++i) {
    fname_[i] = PREFIX_GEN + UtilsString::Int2Str(i);
  }
  cov_ /= REF_SEQ_LEN;
  cout << "coverage: " << cov_ << endl;
  DumpKMerFreq();
}

ReadProcessor::~ReadProcessor() {
}

void ReadProcessor::ProcessFiles() {
  return;
  // We can only open a limited # of files at a time. Thats why we need
  // multiple passed to make these files.
  int file_inc = 1000;
  for (int i = 0; i < fc_;) {
    int s = i;
    int e = s + file_inc;
    if (e > fc_) e = fc_ - 1;
    ProcessWithinValues(s, e);
#ifdef DEBUG
    cout << "finished processing files" <<  s << '-' <<  e << endl;;
#endif  // DEBUG
    i = e + 1;
  }
}

void ReadProcessor::ProcessWithinValues(int from, int to) {
  from_val_ = from;
  to_val_ = to;

  for (int i = from_val_; i <= to_val_; ++i) {
    fp_[i] = new fstream(fname_[i].c_str(), ios::out | ios::binary);
    frecs_[i] = 0;
  }
  for (int i = 0; i < raw_read_.size(); ++i) {
    ProcessPair(raw_read_[i]);
  }
  for (int i = from_val_; i <= to_val_; ++i) {
    fp_[i]->close();
  }
}

const string ReadProcessor::GetID(int id) {
  return raw_read_[id].Lid() + raw_read_[id].Rid();
}

const string& ReadProcessor::GetLMar(int id, mar_types mt) const {
  //  mar_types get_mar = ((mt == LMAR || mt == LMARRC) : (mt + 1) ? (mt - 1));
  // if (mt == RMAR || mt == RMARRC) {
  return raw_read_[id].Mar(mar_type_pair[mt]);
    // }
}

const string& ReadProcessor::GetRMar(int id, mar_types mt) const {
  return raw_read_[id].Mar(mt);
}

const string& ReadProcessor::GetMar(int id, mar_types mt) const {
  return raw_read_[id].Mar(mt);
}

int ReadProcessor::GetMarLen(int id, mar_types mt) const {
  return raw_read_[id].Mar(mt).size();
}

void ReadProcessor::FreqCount(const string& str) const {
  for (int i = 0; i + HASH_LEN <= str.size(); ++i) {
    int val = UtilsString::String2Int(str, i, HASH_LEN);
    ++fmer_[val];
  }
}

const IntIntMtTripleVec& ReadProcessor::GetRecords(int val) {
  LoadFile(val);
  return inmem_files_[val];
}

void ReadProcessor::ProcessPair(const RawRead& rread) {
  int id = rread.id();
  ProcessRead(id, rread.Mar(LMAR), RMAR);
  ProcessRead(id, rread.Mar(LMARRC), RMARRC);
  ProcessRead(id, rread.Mar(RMAR), LMAR);
  ProcessRead(id, rread.Mar(RMARRC), LMARRC);
}

void ReadProcessor::ProcessRead(int id, const string& anchor,
				mar_types mt) {
  int val = UtilsString::String2Int(anchor, 0, HASH_LEN);
  Add2IndexedFile(val, id, 0, mt);
  for (int i = 1; i + HASH_LEN <= (int)anchor.size(); ++i) {
    val = UtilsString::RightShiftStrVal(anchor[i - 1], anchor[i + HASH_LEN - 1],
					val, HASH_LEN);
    Add2IndexedFile(val, id, i, mt);
  }
}

void ReadProcessor::Add2IndexedFile(int val, int id, int offset,
				    mar_types mt) {
  if (val < from_val_ || val > to_val_) return;
  IntIntInt t;
  t.a = id;
  t.b = offset;
  t.c = mt;
  fp_[val]->write((char*)(&t), sizeof(t));
  ++frecs_[val];
}

void ReadProcessor::LoadFile(int val) {

  // File already loaded.
  if (inmem_files_[val].size()) return;

//   // See whether we have enough memory left to load this file, otherwise remove
//   // some.
//   // If the amount of free memory is less than 50%, unload some files.
//   for (int i = 0; i < fc_; ++i) {
//     if (!inmem_files_[i].size()) continue;
//     struct sysinfo sinfo;
//     sysinfo(&sinfo);
//     if (sinfo.freeram > sinfo.totalram / 20) {
//       break;
//     }
// //     if (sinfo.freeram - frecs_[val] * 6 > sinfo.totalram / 10) {
// //       break;
// //     }
// #ifdef DEBUG
//     cout << "unloading file: " << sinfo.freeram << ' ' << i << endl;
// #endif  // DEBUG
//     //    inmem_files_[i].clear();
//     IntIntMtTripleVec().swap(inmem_files_[i]);
//   }

  // Load the new file.
  fstream fp(fname_[val].c_str(), ios::in | ios::binary);
  while (!fp.eof()) {
    IntIntInt t;
    fp.read(reinterpret_cast<char*>(&t), sizeof(t));
    if (fp.fail()) break;
    if (ConsumedBy(t.a) != -1) continue;
    inmem_files_[val].push_back(IntIntMtTriple(t.a, t.b, (mar_types)t.c));
  }
  fp.close();
}

// void ReadProcessor::DumpFile(int val) const {
//   cout << "dumping file " << val << endl;
//   fstream* fp = UtilsFile::OpenFile(Fname_[val], "r");
//   int id, offset;
//   string lmar, rmar;
//   while ((*fp) >> id >> offset >> lmar >> rmar) {
//   //while (!fp->eof()) {
//     //IntIntStrStrQuad frec(-1, -1, "", "");
//     //fp->read((char*)(&frec), sizeof(frec));
//     cout << id << ' ' << offset << ' ' << lmar << ' ' << rmar << endl;
//     //    cout << frec.first << ' ' << frec.second << ' ' << frec.third << ' '
//     // << frec.fourth << endl;
//   }
//   UtilsFile::CloseFile(fp);
// }

void ReadProcessor::DumpKMerFreq() const {
  for (int i = 0; i < fc_; ++i) {
    cout << i << " " << fmer_[i] << endl;
  }
}

int ReadProcessor::TotalPairs() const { 
  return raw_read_.size();
}
