// This is the driver program that assembles harvard reads by invoking 
// appropriate method from assembler.h, etc.
#include <iostream>
#include <fstream>

#include "assembler.h"
//#include "macros.h"
//#include "constants.h"
#include "global.h"
#include "raw-read.h"
#include "seeded-read.h"
#include "header.h"
#include "read-processor.h"
#include "utils-file.h"
#include "utils-string.h"

using namespace std;

// These are the seeded reads.
vector<SeededRead> sreads;

ReadProcessor* read_processor;

bool GetFirstLine(fstream& fin, char* str = NULL);
bool ReadSeededFastaFile(const string& fname, vector<SeededRead>* sreads);

void PrintConfigInfo();

int main(int argc, char* argv[]) {
//   if (argc != 6) {
//     cout << "usage: shorty-assembler -o overlap seeds.fasta reads.fasta reads.qual"
//       ".ext";
//     exit(1);
//   }

  // Read the arguments.
  int c;
  while ((c = getopt(argc, argv, "r:o:s:l:L:t:")) != -1) {
    switch (c) {
    case 'r':
      Global::MAX_READ_USE = atoi(optarg);
      break;
    case 'o':
      Global::MIN_OVERLAP = atoi(optarg);
      Global::MIN_SCORE = 4 * Global::MIN_OVERLAP -
	min((int)(0.25 * 4 * Global::MIN_OVERLAP), 25);
      break;
    case 's':
      Global::SUB_2MAP_SEED = atoi(optarg);
      break;
    case 'l':
      Global::DESIRED_CONTIG_LEN = atoi(optarg);
      break;
    case 'L':
      Global::REF_SEQ_LEN = atoi(optarg);
      break;
    case 't':
      Global::DESIRED_THICKNESS = atoi(optarg);
      break;
    default:
      cout << "unknown option!!!\n";
      break;
    }
  }
//  PrintConfigInfo();
  assert(optind + 3 == argc);
  const string& ext = argv[optind + 2];
  cout << "using output extension: " << ext << endl;
  fstream* log = UtilsFile::OpenFile(PREFIX_OUT + "log" + ext, "w");
  fstream* contigs_out = UtilsFile::OpenFile(PREFIX_OUT + "contigs" + ext, "w");
  fstream* geo_out = UtilsFile::OpenFile(PREFIX_OUT + "geo" + ext, "w");
  fstream* geom_out = UtilsFile::OpenFile(PREFIX_OUT + "geom" + ext, "w");
  fstream* bambus_out = UtilsFile::OpenFile(PREFIX_OUT + "bambus-in.contig" +
					    ext, "w");
  cout << "using seed file: " << argv[optind] << endl;
  ReadSeededFastaFile(argv[optind], &sreads);
  cout << "using read file: " << argv[optind + 1] << endl;
  read_processor = new ReadProcessor(argv[optind + 1]);
//  PrintConfigInfo();
    
  // Initialize the assembler.
  Assembler assembler(sreads, read_processor, log, contigs_out,
		      geo_out, geom_out, bambus_out);

  PrintConfigInfo();
  // Add the seeds we are interested in.
  for (int i = 0; i < sreads.size(); ++i) {
    assembler.AddSRead(i);
  }
  *log << "finished adding seeded reads to the assembler...\n";
  if (assembler.Assemble()) {
    cout << "assembling finished...\n";
  } else {
    cout << "assembling failed...\n";
  }
  UtilsFile::CloseFile(log);
  UtilsFile::CloseFile(contigs_out);
  UtilsFile::CloseFile(geom_out);
  delete read_processor;
  return 0;
}

bool GetFirstLine(fstream& fin, char* str) {
  char c;
  int i = 0;
  while ((c = fin.get()) != '\n' || !i) {
    if (c == EOF) break;
    if (str) str[i] = c;
    if (c != '\n') ++i; 
  }
  if(str) str[i] = 0;
  return i ? true : false;
}

bool ReadSeededFastaFile(const string& fname, vector<SeededRead>* sreads) {
  fstream fin(fname.c_str(), ios::in);
  int id = 0;  // Id should come as input, but at this point stick to this.

  // No error handling at this point.
  while (GetFirstLine(fin)) {

    // At present, doing nothing with the id.
    string str, strrc;
    fin >> str;
    //str = str.substr(0, str.size() / USE_SEED_PORTION);
    if (str[0] == '>') continue;
    transform(str.begin(), str.end(), str.begin(), ::toupper);
#ifdef COMPRESS
    str = UtilsString::CompressStr(str);
#endif
    strrc = UtilsString::ReverseComplement(str);
#ifdef COLOR_SPACE
    str = UtilsString::Fasta2Solid(str);
    strrc = UtilsString::Fasta2Solid(strrc);
#endif

    // CAUTION: this 100 should be replaced by coverage.
    const ShortyString& a = ShortyString(str, 100);
    sreads->push_back(SeededRead(a, id, 0));
    ++id;
// #ifndef SINGLE_STRAND  // not in single strand mode.
//     const ShortyString& b = ShortyString(strrc, 100);
//     sreads->push_back(SeededRead(b, id, id - 1, 0));
//     ++id;
// #endif
  }
  fin.close();
  return true;
}

void PrintConfigInfo() {
  cout << "running with the following configurations:\n"
       << "overlap between reads\t\t\t" << Global::MIN_OVERLAP << endl
       << "min score for overlap\t\t\t" << Global::MIN_SCORE << endl
       << "substitution error allowed per read\t" << Global::SUB_2MAP_SEED << endl
       << "output thickness\t\t\t" << Global::DESIRED_THICKNESS << endl
       << "min contig len\t\t\t\t" << Global::DESIRED_CONTIG_LEN << endl
       << "reference len\t\t\t\t" << Global::REF_SEQ_LEN << endl
       << "read use\t\t\t\t" << Global::MAX_READ_USE << endl;
}
