#include <fstream>

#include "contig.h"
#include "read-processor.h"

// Contig::Contig(ReadProcessor* rp)
//   : rp_(rp) {
//   min_offset_ = min_gen_ = INF;
//   max_offset_ = max_gen_ = -INF;
// }

Contig::Contig(bool right, int seed_id, int seed_len, const ShortyString& str,
	       const ReadProcessor* rp)
  : right_(right), seed_id_(seed_id), seed_len_(seed_len), str_(str), rp_(rp) {
  min_offset_ = min_gen_ = INF;
  max_offset_ = max_gen_ = -INF;
}

Contig::Contig(bool right, int seed_id, int seed_len, const ShortyString& str,
	       int id, const ReadProcessor* rp)
  : right_(right), seed_id_(seed_id), seed_len_(seed_len), str_(str), id_(id),
    rp_(rp) {
  min_offset_ = min_gen_ = INF;
  max_offset_ = max_gen_ = -INF;
}

Contig::Contig(const ShortyString& str) {
  str_ = str;
}

Contig Contig::SubContig(int start, int len) const {
}

void Contig::AddRead(int id, int offset, int gen, mar_types mt) {
  reads_.push_back(id);
  offsets_.push_back(offset);
  //  seed_offsets_.push_back(seed_offset);
  mar_types_.push_back(mt);
  min_offset_ = min(min_offset_, offset);
  max_offset_ = max(max_offset_, offset);
  min_gen_ = min(min_gen_, gen);
  max_gen_ = max(max_gen_, gen);
}

void Contig::PrintContigFormat(fstream* out) const {
  *out << "##asm_" << id_ << " " << Size() << " " << str_.size()
       << " bases, 00000000 checksum.\n" << str_ << endl;
  for (int i = 0; i < reads_.size(); ++i) {
    mar_types mt = mar_types_[i];
    int id = reads_[i];
    int offset = offsets_[i];
    const string& mar = rp_->GetMar(id, mt);
    *out << "#r" << id << "." 
	 << ((mt == LMAR || mt == LMARRC) ? 'x' : 'y') << '(' << offset
	 << ") [" << ((mt == LMARRC || mt == RMARRC) ? "RC] " : "] ")
	 << mar.size() << " bases, 00000000 checksum. {";
    if (mt == LMAR || mt == RMAR) {
      *out << "1 " << mar.size() <<  "} <" << offset + 1 << " "
	   << offset + mar.size() << ">\n";
    } else {
      *out << mar.size() <<  " 1} <" << offset + mar.size()
	   << " " << offset + 1 << ">\n";
    }
    *out << mar << endl;
  }
}

void Contig::PrintGeoFormat(fstream* out) const {
  *out << id_ << " " << reads_.size() << endl;
  for (int i = 0; i < reads_.size(); ++i) {
    mar_types mt = mar_types_[i];
    int id = reads_[i];
    int offset = offsets_[i];    
    *out << id << " " << offset << " " << mt << " ";  
  }
  *out << endl;
}

void Contig::PrintFastaFormat(fstream* out) const {
  *out << ">contig_asm_" << id_ << endl;
  *out << str_ << endl;
}

bool Contig::IsGoodSeed(fstream* out, const Contig& c) const {
  return true;
  *out << ">contig_asm_" << id_ << endl;
  *out << str_ << endl;
  vector<int> thickness(str_.size());
  for (int i = 0; i < reads_.size(); ++i) {
    int o = offsets_[i];
    int s = rp_->GetMarLen(reads_[i], mar_types_[i]);
    for (int j = 0; j < s; ++j) {
      ++thickness[o + j];
    }
  }
  for (int i = 0; i < c.Size(); ++i) {
    int o = c.GetOffset(i);
    int s = rp_->GetMarLen(c.GetRead(i), c.GetMt(i));
    for (int j = 0; j < s; ++j) {
      ++thickness[o + j];
    }
  }
  int m = -INF;
  for (int i = 0; i < str_.size(); ++i) {
    m = max(m, thickness[i]);
  }
  *out << "max thickness: " << m << endl;
  for (int i = 0; i < str_.size(); ++i) {
    *out << thickness[i] << " ";
  }
  *out << endl;
  //  return m <= rp_->Coverage();
  return m <= 150;
}

void Contig::Add(const Contig& c) {
  for (int i = 0; i < c.Size(); ++i) {
    reads_.push_back(c.GetRead(i));
    offsets_.push_back(c.GetOffset(i));
    //seed_offsets_.push_back(c.GetSeedOffset(i));
    mar_types_.push_back(c.GetMt(i));
  }
  //  min_gen_ = min(min_gen_, c.GetMinGen());
  // max_gen_ = max(max_gen_, c.GetMaxGen());
  //min_offset_ = min(min_offset_, c.GetMinOffset());
  //max_offset_ = max(max_offset_, c.GetMaxOffset());
}

// void Contig::ShiftOffsets(int x) {
//   min_offset_ += x;
//   max_offset_ += x;
//   for (int i = 0; i < offsets_.size(); ++i) {
//     offsets_[i] += x;
//   }
// }

// IntPair Contig::GapWindow() const {
//   int l = -INF, r = INF;
//   int l0 = -INF, r0 = INF;

//   // We will take the intersection of the windows offered by each read to strip
//   // down our window: [l, r].
//   for (int i = 0; i < reads_.size(); ++i) {
//     // WRONG: we are assuming that the left and right mars are of same size,
//     // at least it is true for AB.
//     // For a right contig: I - (S - i) - j - R
//     // For a left contig: I - (C - j) - i - R
//     int p, q;
//     p = q = INSERT_LEN - rp_->GetMarLen(reads_[i], mar_types_[i]);
    
//     // right
//     p = p - (seed_len_ - seed_offsets_[i]) - offsets_[i];
      
//     // left
//     q = q - (str_.size() - offsets_[i]) - seed_offsets_[i];
//     l = max(l, p - INSERT_LEN_DEV);
//     l0 = max(l0, q - INSERT_LEN_DEV);
//     r = min(r, p + INSERT_LEN_DEV);
//     r0 = min(r0, q + INSERT_LEN_DEV);
//   }
//   if (l > r) swap(l, r);
//   if (l0 > r0) swap(l0, r0);
//   return IntPair(min(l, l0), max(r, r0));
// }
