#include <iostream>
#include <cassert>

#include "hash.h"
#include "utils-string.h"
#include <cstring>

Hash::Hash(int len) {
  hash_len_ = len;
  
  // The table size should be 4^len.
  table_.resize(1 << (2 * len));
  memset(char_val_, 0, sizeof(char_val_));
  char_val_['A'] = char_val_['a'] = 0;
  char_val_['T'] = char_val_['t'] = 1;
  char_val_['G'] = char_val_['g'] = 2;
  char_val_['C'] = char_val_['c'] = 3;
}

Hash::~Hash() {
  table_.resize(0);
}

int Hash::HashVal(const string& str, int start) const {
  if (start < 0 || start + hash_len_ > str.length()) {
    //cout << "invalid value for 'start':" << start << " " << hash_len_ << " " << str.length() << endl;
    return -1;
  }
//   int total = 0;
//   for (int i = 0; i < str.length() && i < hash_len_; ++i) {
//     const string char_pool = "ATGCUatgcu?Nn";
//     if (char_pool.find(str[i + start]) == string::npos) {
//       cout << "invalid character found" << str[i + start] << endl;
//       //return -1;
//       continue; // For 'Helicos' we have other characters...
//     }
//     total += (1 << (2 * i)) * char_val_[str[i + start]];
//   }
//   return total;
  return UtilsString::String2Int(str, start, hash_len_);
}

int Hash::LeftShiftHashVal(char gone, char come, int prev) const {
  int newval = ((prev - char_val_[gone]) >> 2) + 
    (1 << (2 * (hash_len_ - 1))) * char_val_[come];
  assert(newval >= 0);
  return newval;
}

int Hash::Insert(const string& str, int keya, int keyb, int start) {
  int val = HashVal(str, start);
  if (val == -1) return -1;
  list<IntPair>::const_iterator it = table_[val].begin();
  //for (int i = 0; i < table_[val].size(); ++i) {
  for (; it != table_[val].end(); ++it) {
    if (it->first == keya && it->second == keyb) {
    //if (table_[val][i] == pair<int, int> (keya, keyb)) {
      //      cout << "value already exists\n";
      return -2;
    }
  }
  table_[val].push_back(pair<int, int> (keya, keyb));
  return val;
}

int Hash::Insert(const string& str, int keya, int keyb, int start, int prev) {
  assert(start > 0);
  int val = LeftShiftHashVal(str[start - 1], str[start + hash_len_ - 1], prev);
  if (val == -1) return -1;
  list<IntPair>::const_iterator it = table_[val].begin();
  //for (int i = 0; i < table_[val].size(); ++i) {
  for (; it != table_[val].end(); ++it) {
    if (it->first == keya && it->second == keyb) {
      //cout << "value already exists\n";
      return -2;
    }
  }
  table_[val].push_back(pair<int, int> (keya, keyb));
  return val;
}

const list<IntPair>& Hash::GetBucket(int id) const {
  assert(id >= 0 && id < table_.size());
  return table_[id];
}
