#ifndef MACROS_H
#define MACROS_H

#include "header.h"

// Define if we want debug mode enabled. Warning: this might significantly slow
// down the assembler.
#define DEBUG 0

// Whether we should re-use contigs or not.
#define REUSE_CONTIG 1

// Define whether to remove the ends of a contig. This is used in later stages
// of the assembler (e.g. geo-driver, merger).
#define CUT_TAILS 1

#define NO_UNKNOWN 1

// Use color space data, used for AB.
#define COLOR_SPACE 1

// This is to enable compression for Helicos data in an attempt to get rid of
// homo-polymar problem.
//#define COMPRESS 1

// How many bases should we use to hash a read/sequence
const int HASH_LEN = 8;//7;;

// Minimum length of a read.
const int MIN_READ_LEN = 10;//15;// 20;//10;

// Minimum # of bases to consider for overlap.
//const int MIN_OVERLAP = 15;//75;//50;//75;// 50;//15;//10;//15;//50;//25;//15;//10;//25;//15;//85;//50;//20;

// Minimum consequtive overlaps needed for an overlap. This is to prevent porous
// overlaps.
const int MIN_CONSEQ_OVERLAP = 8;//9;//12;//9;

// Minimum score that will make an overlap eligible to merge.
//const int MIN_SCORE = 40;//280;//185;//280;//185;//40;//30;//40;//150;//75;//40;

// This determines the amount of score that should come as a result of direct
// match. This restricts too many deletions/substitutions.
const int MIN_SCORE2OVERLAP = 0;//75;//70;//50;//75;

// A base should be taken confidently if it has at least this percentage of
// votes as its neighbour.
const int BASE_CONFIDENCE = 30;

// This can be used to bypass the constraint set by 'MIN_OVERLAP'. This is
// useful for smaller reads, e.g. length < MIN_OVERLAP.
const int PERCENTAGE_LEN4OVERLAP = 80;

// Max lenght for which we can try overlap.
const int MAX_LEN4OVERLAP = 8000;

// Min length for which we allow deletion.
const int MIN_LEN4DELETION = 6;

// Min length of contigs which will be used by 'shorty-geography'.
const int MIN_LEN4GEO = 50;

// Percentage of deletion allowed.
const int PERCENTAGE_DEL_ALLOWED = 0;//2;//1;//4;//2;//1;//2;//5;//10;//2

// Percentage of substitution allowed.
const int PERCENTAGE_SUBS_ALLOWED = 20;

// Co-efficient to be multiplied with 'match' scores.
const int COEFF_MATCH_SCORE = 2;

// Co-efficient to be multiplied with 'delete' scores.
const int COEFF_DEL_SCORE = 5;//10;//4;//5; //7

// Co-efficient to be multiplied with 'substitution' scores.
const int COEFF_SUBS_SCORE = 5;//8;//100;//4;//100;//5; //8

// Specify the length which will qualify a contig to be used as a seed.
const int SEED_LEN = 100;//50; //100;  //30; 

// Average read length, an estimated value is ok.
//const int AVG_READ_LEN = 26;

// # of deletions allowed to map a read in a seed. In future it can be
// percentage.
const int DEL_2MAP_SEED = 0;//1;

// # of substitutions allowed to map a read in a seed. In future it can be
// percentage.
//const int SUB_2MAP_SEED = 3;//4;//3;//0;

// Deviation allowed for generation count while trying to merge.
const int GEN_DEV = 3;//5;//2;//5;

// This is a cut-off lenght for contigs. Anything below this will be ignored.
// This is sometimes used by the merger, vizualizer, etc.
const int CUT_OFF_LEN = 50;

// This is the percentage of how much we cut from the two ends of a contig. This
// is useful in the later stages when we want to merge long contigs but can't
// proceed due to tail errors.
const int PERCENTAGE_CUT_CONTIG_TAIL = 50;

// Prefix to add before all output files starting from root.
const string PREFIX_OUT = "out/";

// Desired thickness we want in the output.
//const int DESIRED_THICKNESS = 10;//3;//4;//5;

// Desired contig len we want in the output.
//const int DESIRED_CONTIG_LEN = 100;//30;//50;//100;

// Desired max contig len we want in the output.
const int DESIRED_CONTIG_LEN_MAX = 2000;

// Desired overlaps of contigs in the output.
const int DESIRED_CONTIG_OVERLAP = 100;

// Masimum # of time a read can be used.
//const int MAX_READ_USE = 7;//3;//4;//5;//2;

// Reference sequence length.
//const int REF_SEQ_LEN = 2007491;

#endif  //  MACROS_H
