#ifndef OVERLAP_GRAPH_NODE_H
#define OVERLAP_GRAPH_NODE_H

#include "short-string.h"
#include "header.h"

// A node in the overlap graph.
class OverlapGraphNode {
 public:
  // Do nothing constructor.
  OverlapGraphNode() {}

  // Only initialize the string.
  OverlapGraphNode(const ShortyString& str) : str_(str) {}

  // Initialize the string and single id.
  OverlapGraphNode(const ShortyString& str, int id, int offset) : str_(str)
    { ids_.push_back(IntPair(id, offset)); }

  // Initialize everything.
  OverlapGraphNode(const ShortyString& str, const IntPairVec& ids) :
    str_(str), ids_(ids) {}

  // Getter methods.
  const ShortyString& Str() const { return str_; }
  const IntPairVec& Ids() const { return ids_; }
  const int CountIds() const { return ids_.size(); }

  // Setter/mutator methods.
  // Adding an id to 'ids_'.
  //  void AddIdOffset(int id, int offset) { ids_.push_back(id); }
  
  // Removing an id from 'ids_'.
  //  void RemId(int id) { ids_.erase(find(ids_.begin(), ids_.end(), id)); }

  // Clear the list of Ids....
  void ResetIds() { ids_.clear(); }

 private:
  // Contig represented by this node.
  ShortyString str_;
 
  // Ids and offsets of the reads that are comprising this node. Note that the
  // offset is the position of the anchor read in the seeded read.
  //IntVec ids_;
  IntPairVec ids_;
};

#endif  //  OVERLAP_GRAPH_NODE_H
