#ifndef READ_PROCESSOR_H
#define READ_PROCESSOR_H

#include "global.h"
#include "header.h"
#include "macros.h"

class TrieNode;
class RawRead;
class Trie;

// This will handle all read related queries and manage the I/O.
class ReadProcessor {
 public:
  ReadProcessor(const string& path);
  ~ReadProcessor();
  const string GetID(int id);
  //  const string& GetLMar(int id, mar_types mt) const;
  //const string& GetRMar(int id, mar_types mt) const;
  const string& GetMar(int id, mar_types mt) const;
  IntIntMtTripleVec GetRecords(const string& str) const;
  bool ReadInTrie(const string& str) const;
  int GetMarLen(int id, mar_types mt) const;

  inline int ConsumedBy(int i) const { return used_[i]; }
  inline int Coverage() const { return cov_; }
  inline void Consume(int i, int seed_id) {
    //prev_used_[i] = used_[i];
    used_[i] = seed_id;
    if (num_used_[i] < Global::MAX_READ_USE) {
      ++num_used_[i];
    }
  }
  //  inline void Release(int i) { used_[i] = prev_used_[i]; --num_used_[i]; }
  //inline void Reset(int i) { used_[i] = prev_used_[i] = -1; num_used_[i] = 0; }
  inline int NumUsed(int i) { return num_used_[i]; } 
  int TotalPairs() const;
  //  inline int UnusedPairs() const { return total_ - used_.count(); }
  void Debug() const;

 private:
  int LocateRead(const string& read) const;  

  static int cov_, total_;//, from_val_, to_val_;

  // Check whether a particular pair is used or not.
  //  static bitset<NUM_PAIR> used_;
  //  static int used_[NUM_PAIR];
  //prev_used_[NUM_PAIR];
  //static char num_used_[NUM_PAIR];
  static int* used_;
  static char* num_used_;

  //static vector<const char*> all_reads_c;
  static vector<string> all_reads_;
  static vector<RawRead> raw_read_;
  //Trie* trie_;
  TrieNode* trie_;
};

#endif  //  READ_PROCESSOR_H
