#include "trie.h"
#include "utils-string.h"

int Trie::node_count_ = 0;

void Trie::Insert(const string& str, int id, mar_types mt) {
  TrieNode* curr = &root_;
  for (int i = 0; i < str.size(); ++i) {
    base_types bt = UtilsString::Char2Bt(str[i]);
    if (!curr->children_[bt]) {
      curr->children_[bt] = new TrieNode;
      ++node_count_;
    }
    curr = curr->children_[bt];
  }
  if (!curr->leaf_info_) {
    curr->leaf_info_ = new ListNode(id, mt);
  } else {
    curr->leaf_info_->Insert(id, mt);
  }
  //  curr->leaf_info_->push_back(IntMtPair(id, mt));
}

IntMtPairVec Trie::Search(const string& str, int offset,
			  int error_allowed) const {
  IntMtPairVec ret_info;
  VisitNode(str, offset, error_allowed, &root_, &ret_info);
  return ret_info;
}

// Not handling indel erro right now.
void Trie::VisitNode(const string& str, int pos, int rem_error,
		     const TrieNode* node, IntMtPairVec* ret_info) const {
  if (!node) return;
  if (node->leaf_info_) {
    //    IntMtPairVec::const_iterator it = node->leaf_info_->begin();
    //for (; it != node->leaf_info_->end(); ++it) {
      //      if (rem_error < SUB_2MAP_SEED) cout << "found with error!!\n";
    //  ret_info->push_back(*it);
    // }
    ListNode* start = node->leaf_info_;
    while (start) {
      ret_info->push_back(IntMtPair(start->id_, start->mt_));
      start = start->next_;
    }
  }
  if (pos == str.size()) return;
  for (int i = 0; i < BASE_TYPES; ++i) {
    if (node->children_[i]) {
      if (i == UtilsString::Char2Bt(str[pos])) {
	VisitNode(str, pos + 1, rem_error, node->children_[i], ret_info);
      } else if (rem_error) {
	VisitNode(str, pos + 1, rem_error - 1, node->children_[i], ret_info);
      }
    }
  }
}

IntMtPair Trie::Search(const string& str) const {
  return VisitNode(str, 0, &root_); 
}

IntMtPair Trie::VisitNode(const string& str, int pos,
			  const TrieNode* node) const {
  if (pos == str.size()) {
    if (node->leaf_info_) {
      return IntMtPair(node->leaf_info_->id_, node->leaf_info_->mt_); 
    }
    return IntMtPair(-1, LMAR);
  }
  int i = UtilsString::Char2Bt(str[pos]);
  if (node->children_[i]) {
    return VisitNode(str, pos + 1, node->children_[i]);
  }
  return IntMtPair(-1, LMAR);
}
