#include <cstdio>
#include <fstream>
#include <iterator>
#include <iostream>
#include <set>

#include "kmar-table.h"
#include "utils-string.h"
#include "visualize.h"

string org, rorg;

// These are the tables used by kmar-tables. Declared here to make them global.
IntVec kmar_table[1 << (KMAR_LEN << 1)];
IntVec kmar_rc_table[1 << (KMAR_LEN << 1)];

void CallPrint(const string& f_org, const string& f_seeds,
	       const string& f_contigs, const string& f_out, bool read_pos);

int main(int argc, char** argv) {
  if (argc != 6) {
    cout << "usage: [org, seeds, contigs, output, read_pos]\n";
    exit(1);
  }
  string arg = argv[5];
  CallPrint(argv[1], argv[2], argv[3], argv[4], arg == "true" ? true
	    : false);
  return 0;
}

void CallPrint(const string& f_org, const string& f_seeds,
	       const string& f_contigs, const string& f_out, bool read_pos) {
  fstream forg(f_org.c_str(), ios::in);
  fstream fseeds(f_seeds.c_str(), ios::in);
  fstream fcontigs(f_contigs.c_str(), ios::in);
  fstream fout(f_out.c_str(), ios::out);
  string org, orgrc;
  forg >> org;
  orgrc = UtilsString::ReverseComplement(org);
#ifdef COLOR_SPACE
  org = UtilsString::Fasta2Solid(org);
  orgrc = UtilsString::Fasta2Solid(orgrc);
#endif
  
  //cout << org << endl;
  //  cout << orgrc << endl;
  Visualize* viz = new Visualize(org, &fout);
  Visualize* vizrc = new Visualize(orgrc, &fout);

  KmarTable ktable(org, kmar_table);
  KmarTable ktablerc(orgrc, kmar_rc_table);

  string str, strrc;
  int toffset;
  int total = 0;
  int match = 0;
  int nt_total = 0;
  int nt_match = 0;

  // Map the seeded reads first.
  while (fseeds >> str) {
    if (str[0] == '>') continue;
    strrc = UtilsString::ReverseComplement(str);
#ifdef COMPRESS
    str = UtilsString::CompressStr(str);
    strrc = UtilsString::CompressStr(strrc);
#endif
#ifdef COLOR_SPACE
    str = UtilsString::Fasta2Solid(str);
    strrc = UtilsString::Fasta2Solid(strrc);
#endif
    string printa, printb, print;
    int scorea, scoreb, posa, posb;
    int pos;
    int match, ins, hins, del, hdel;
    posa = UtilsString::FindBestPos(org, str, ktable, &scorea, &match, &ins,
				    &hins, &del, &hdel, &printa);
    posb = UtilsString::FindBestPos(orgrc, str, ktablerc, &scoreb, &match, &ins,
				    &hins, &del, &hdel, &printb);
    //    posa = FindBestPos(org, str, ktable, &scorea, &printa);
    //posb = FindBestPos(orgrc, str, ktablerc, &scoreb, &printb);
    if (posa == -1 && posb == -1) {
      cout << "could not map: " << str << endl;
      continue;
    }
    if (scorea > scoreb) {
      pos = posa;
      print = printa;
    } else {
      pos = posb;
      print = printb;
    }
    for (int i = 0; i < print.size(); ++i) {
      print[i] = '+';
    }
    if (scorea > scoreb) {
      viz->AddContig(print, pos);
    } else {
      vizrc->AddContig(print, pos);
    }

    // Map the RC of the seeded read.
    //    str = UtilsString::ReverseComplement(str);
    //    int match, ins, hins, del, hdel;
    posa = UtilsString::FindBestPos(org, strrc, ktable, &scorea, &match, &ins,
				    &hins, &del, &hdel, &printa);
    posb = UtilsString::FindBestPos(orgrc, strrc, ktablerc, &scoreb, &match, &ins,
				    &hins, &del, &hdel, &printb);
    // posa = FindBestPos(org, strrc, ktable, &scorea, &printa);
    //posb = FindBestPos(orgrc, strrc, ktablerc, &scoreb, &printb);
    if (posa == -1 && posb == -1) {
      cout << "could not map: " << strrc << endl;
      continue;
    }
    if (scorea > scoreb) {
      pos = posa;
      print = printa;
    } else {
      pos = posb;
      print = printb;
    }
    for (int i = 0; i < print.size(); ++i) {
      print[i] = '+';
    }
    if (scorea > scoreb) {
      viz->AddContig(print, pos);
    } else {
      vizrc->AddContig(print, pos);
    }
  } 
  //  cout << "done seed\n";
  string left, right;
  int flag = true;
  int temp_counter = -1;
  int total_mismatch = 0;
  string cid;
  while (fcontigs >> str) {
    if (str[0] == '>' || str[0] == '#') {
      //cout << str << endl;
      cid = str;
      continue;
    }
    if (str.size() < 110 || str.size() > 111) continue;
    cout << cid << endl;
    ++temp_counter;
// #ifdef COLOR_SPACE
//     str = UtilsString::Fasta2Solid(str);
// #endif  // COLOR_SPACE
    cout << (++total) << " ";
    string printa, printb, print;
    int scorea, scoreb, posa, posb;
    int pos;
    if (read_pos) {
      fcontigs >> pos;
      if (pos >= 0) {
	viz->AddContig(str, pos);
      } else {
	++pos;
	vizrc->AddContig(str, -pos);
      }
      continue;
    }
    int match, ins, hins, del, hdel;
    posa = UtilsString::FindBestPos(org, str, ktable, &scorea, &match, &ins,
				    &hins, &del, &hdel, &printa);
    posb = UtilsString::FindBestPos(orgrc, str, ktablerc, &scoreb, &match, &ins,
				    &hins, &del, &hdel, &printb);
    if (posa == -1 && posb == -1) {
      cout << "could not map: " << str << endl;
      total_mismatch += str.size();
      continue;
    }
    if (scorea > scoreb) {
      pos = posa;
      print = printa;
    } else {
      pos = posb;
      print = printb;
    }
    if (scorea > scoreb) {
      viz->AddContig(print, pos);
    } else {
      vizrc->AddContig(print, pos);
    }
    if (scorea <= scoreb) {
      cout << "- ";
    } else {
      cout << "+ ";
    }
    cout << pos << " " << pos + str.size() - 1 << " " << str << " " << print
	 << endl;
    for (int i = 0; i < print.size(); ++i) {
      if (print[i] >= 'a' && print[i] <= 'z') ++total_mismatch;
    }
  }
  viz->Print();
  vizrc->Print();
  cout << "total bad bases: " << total_mismatch << endl;
}

// int FindBestPos(const string& a, const string& b, const KmarTable& ktable,
// 		int* score, string* c) {
//   int best_score = -INF;
//   int best_pos = -1;
//   string best_str = "";
//   string ret;
//   IntPairVec pos = ktable.GetGluedPos(b);
//   IntPairVec::const_iterator it = pos.begin();
//   int last = -1;
//   for (; it != pos.end(); ++it) {
// //     int x = it->first - it->second - b.size();
// //     if (x < 0) x = 0;
// //     if (x <= last) x = last + 1; 
// //     int y = it->first - it->second + b.size() / 10;
// //     if (y >= a.size()) y = a.size() - 1;
// //     last >?= y;
// //     //     for (int i = x; i <= y; ++i) {
// //     //       int len = b.size() + b.size() / 5;
// //     //       if (i + len > a.size()) len = a.size() - i + 1;
// //     //       int t = UtilsString::AlignWithReference(i, len, a, b, &ret);
// //     //       if (t >= best_score) {
// //     // 	best_score = t;
// //     // 	best_pos = i;
// //     // 	best_str = ret;
// //     //       } else if (t == best_score && ret.size() < best_str.size()) {
// //     // 	best_pos = i;
// //     // 	best_str = ret;
// //     //       }
// //     //     }
//     int x = it->first - it->second;
//     if (x <= last) continue;
//     last = x;
//     x -= (int)b.size() / 3;
//     if (x < 0) x = 0;  // give some room.
//     int tscore, tpos, tmatch, tins, thins, tdel, thdel;
//     string tc;
//     int len = b.size() + b.size() + b.size();
//     if (b.size() > 1000) {
//       x = 0;
//       len = a.size();
//     }
//     if (x + len > a.size()) len = a.size() - x + 1;
//     tscore = UtilsString::AlignWithReference(false, x, len, a, b, &tpos, &tmatch,
// 				       &tins, &thins, &tdel, &thdel, &tc);
//     if (tscore > best_score) {
//       best_score = tscore;
//       best_pos = tpos;
//       best_str = tc;
//     }
//   }
//   *c = best_str;
//   *score = best_score;
//   return best_pos;
// }
